use std::{error::Error, fmt::Display};

use crate::rstd::{atomic::au64::*, point::*};

#[derive(Debug)]
pub enum TreeParseError<E> {
    Height(IntParseError),
    Point(PointParseError),
    Key(E),
}

impl<E> From<IntParseError> for TreeParseError<E> {
    fn from(value: IntParseError) -> Self {
        Self::Height(value)
    }
}

impl<E> From<PointParseError> for TreeParseError<E> {
    fn from(value: PointParseError) -> Self {
        Self::Point(value)
    }
}

impl<E: Display> Display for TreeParseError<E> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::Height(height_error) => {
                write!(f, "failed to parse tree height: {height_error}")
            }
            Self::Point(point_error) => {
                write!(f, "failed to parse node reference: {point_error}")
            }
            Self::Key(key_error) => {
                write!(f, "failed to parse node key: {key_error}")
            }
        }
    }
}

impl<E: Error> Error for TreeParseError<E> {}
